[f2fs-dev] [PATCH v4 18/23] nilfs2: Convert nilfs_lookup_dirty_data_buffers() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag(). This change removes 4 calls
to compound_head().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Ryusuke Konishi 
---
 fs/nilfs2/segment.c | 29 -
 1 file changed, 16 insertions(+), 13 deletions(-)

diff --git a/fs/nilfs2/segment.c b/fs/nilfs2/segment.c
index b4cebad21b48..2183e1698f8e 100644
--- a/fs/nilfs2/segment.c
+++ b/fs/nilfs2/segment.c
@@ -680,7 +680,7 @@ static size_t nilfs_lookup_dirty_data_buffers(struct inode 
*inode,
  loff_t start, loff_t end)
 {
struct address_space *mapping = inode->i_mapping;
-   struct pagevec pvec;
+   struct folio_batch fbatch;
pgoff_t index = 0, last = ULONG_MAX;
size_t ndirties = 0;
int i;
@@ -694,23 +694,26 @@ static size_t nilfs_lookup_dirty_data_buffers(struct 
inode *inode,
index = start >> PAGE_SHIFT;
last = end >> PAGE_SHIFT;
}
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
  repeat:
if (unlikely(index > last) ||
-   !pagevec_lookup_range_tag(&pvec, mapping, &index, last,
-   PAGECACHE_TAG_DIRTY))
+ !filemap_get_folios_tag(mapping, &index, last,
+ PAGECACHE_TAG_DIRTY, &fbatch))
return ndirties;
 
-   for (i = 0; i < pagevec_count(&pvec); i++) {
+   for (i = 0; i < folio_batch_count(&fbatch); i++) {
struct buffer_head *bh, *head;
-   struct page *page = pvec.pages[i];
+   struct folio *folio = fbatch.folios[i];
 
-   lock_page(page);
-   if (!page_has_buffers(page))
-   create_empty_buffers(page, i_blocksize(inode), 0);
-   unlock_page(page);
+   folio_lock(folio);
+   head = folio_buffers(folio);
+   if (!head) {
+   create_empty_buffers(&folio->page, i_blocksize(inode), 
0);
+   head = folio_buffers(folio);
+   }
+   folio_unlock(folio);
 
-   bh = head = page_buffers(page);
+   bh = head;
do {
if (!buffer_dirty(bh) || buffer_async_write(bh))
continue;
@@ -718,13 +721,13 @@ static size_t nilfs_lookup_dirty_data_buffers(struct 
inode *inode,
list_add_tail(&bh->b_assoc_buffers, listp);
ndirties++;
if (unlikely(ndirties >= nlimit)) {
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
return ndirties;
}
} while (bh = bh->b_this_page, bh != head);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
goto repeat;
 }
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 04/23] page-writeback: Convert write_cache_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Converted function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag(). This change removes 8 calls
to compound_head(), and the function now supports large folios.

Signed-off-by: Vishal Moola (Oracle) 
Reviewed-by: Matthew Wilcow (Oracle) 
---
 mm/page-writeback.c | 44 +++-
 1 file changed, 23 insertions(+), 21 deletions(-)

diff --git a/mm/page-writeback.c b/mm/page-writeback.c
index 7e9d8d857ecc..aeec8b196232 100644
--- a/mm/page-writeback.c
+++ b/mm/page-writeback.c
@@ -2286,15 +2286,15 @@ int write_cache_pages(struct address_space *mapping,
int ret = 0;
int done = 0;
int error;
-   struct pagevec pvec;
-   int nr_pages;
+   struct folio_batch fbatch;
+   int nr_folios;
pgoff_t index;
pgoff_t end;/* Inclusive */
pgoff_t done_index;
int range_whole = 0;
xa_mark_t tag;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
if (wbc->range_cyclic) {
index = mapping->writeback_index; /* prev offset */
end = -1;
@@ -2314,17 +2314,18 @@ int write_cache_pages(struct address_space *mapping,
while (!done && (index <= end)) {
int i;
 
-   nr_pages = pagevec_lookup_range_tag(&pvec, mapping, &index, end,
-   tag);
-   if (nr_pages == 0)
+   nr_folios = filemap_get_folios_tag(mapping, &index, end,
+   tag, &fbatch);
+
+   if (nr_folios == 0)
break;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch.folios[i];
 
-   done_index = page->index;
+   done_index = folio->index;
 
-   lock_page(page);
+   folio_lock(folio);
 
/*
 * Page truncated or invalidated. We can freely skip it
@@ -2334,30 +2335,30 @@ int write_cache_pages(struct address_space *mapping,
 * even if there is now a new, dirty page at the same
 * pagecache address.
 */
-   if (unlikely(page->mapping != mapping)) {
+   if (unlikely(folio->mapping != mapping)) {
 continue_unlock:
-   unlock_page(page);
+   folio_unlock(folio);
continue;
}
 
-   if (!PageDirty(page)) {
+   if (!folio_test_dirty(folio)) {
/* someone wrote it for us */
goto continue_unlock;
}
 
-   if (PageWriteback(page)) {
+   if (folio_test_writeback(folio)) {
if (wbc->sync_mode != WB_SYNC_NONE)
-   wait_on_page_writeback(page);
+   folio_wait_writeback(folio);
else
goto continue_unlock;
}
 
-   BUG_ON(PageWriteback(page));
-   if (!clear_page_dirty_for_io(page))
+   BUG_ON(folio_test_writeback(folio));
+   if (!folio_clear_dirty_for_io(folio))
goto continue_unlock;
 
trace_wbc_writepage(wbc, inode_to_bdi(mapping->host));
-   error = (*writepage)(page, wbc, data);
+   error = writepage(&folio->page, wbc, data);
if (unlikely(error)) {
/*
 * Handle errors according to the type of
@@ -2372,11 +2373,12 @@ int write_cache_pages(struct address_space *mapping,
 * the first error.
 */
if (error == AOP_WRITEPAGE_ACTIVATE) {
-   unlock_page(page);
+   folio_unlock(folio);
error = 0;
} else if (wbc->sync_mode != WB_SYNC_ALL) {
ret = error;
-   done_index = page->index + 1;
+   done_index = folio->index +
+   folio_nr_pages(folio);
done = 1;
break;
}
@@ -2396,7 +2398,7 @@ int write_cache_pages(struct address_s

[f2fs-dev] [PATCH v4 20/23] nilfs2: Convert nilfs_btree_lookup_dirty_buffers() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag(). This change removes 1 call to
compound_head().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Ryusuke Konishi 
---
 fs/nilfs2/btree.c | 14 +++---
 1 file changed, 7 insertions(+), 7 deletions(-)

diff --git a/fs/nilfs2/btree.c b/fs/nilfs2/btree.c
index b9d15c3df3cc..da6a19eede9a 100644
--- a/fs/nilfs2/btree.c
+++ b/fs/nilfs2/btree.c
@@ -2141,7 +2141,7 @@ static void nilfs_btree_lookup_dirty_buffers(struct 
nilfs_bmap *btree,
struct inode *btnc_inode = NILFS_BMAP_I(btree)->i_assoc_inode;
struct address_space *btcache = btnc_inode->i_mapping;
struct list_head lists[NILFS_BTREE_LEVEL_MAX];
-   struct pagevec pvec;
+   struct folio_batch fbatch;
struct buffer_head *bh, *head;
pgoff_t index = 0;
int level, i;
@@ -2151,19 +2151,19 @@ static void nilfs_btree_lookup_dirty_buffers(struct 
nilfs_bmap *btree,
 level++)
INIT_LIST_HEAD(&lists[level]);
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
 
-   while (pagevec_lookup_tag(&pvec, btcache, &index,
-   PAGECACHE_TAG_DIRTY)) {
-   for (i = 0; i < pagevec_count(&pvec); i++) {
-   bh = head = page_buffers(pvec.pages[i]);
+   while (filemap_get_folios_tag(btcache, &index, (pgoff_t)-1,
+   PAGECACHE_TAG_DIRTY, &fbatch)) {
+   for (i = 0; i < folio_batch_count(&fbatch); i++) {
+   bh = head = folio_buffers(fbatch.folios[i]);
do {
if (buffer_dirty(bh))
nilfs_btree_add_dirty_buffer(btree,
 lists, bh);
} while ((bh = bh->b_this_page) != head);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
 
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 05/23] afs: Convert afs_writepages_region() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert to use folios throughout. This function is in preparation to
remove find_get_pages_range_tag().

Also modified this function to write the whole batch one at a time,
rather than calling for a new set every single write.

Signed-off-by: Vishal Moola (Oracle) 
Tested-by: David Howells 
---
 fs/afs/write.c | 114 +
 1 file changed, 59 insertions(+), 55 deletions(-)

diff --git a/fs/afs/write.c b/fs/afs/write.c
index 9ebdd36eaf2f..c17dbd82a38c 100644
--- a/fs/afs/write.c
+++ b/fs/afs/write.c
@@ -699,82 +699,86 @@ static int afs_writepages_region(struct address_space 
*mapping,
 loff_t start, loff_t end, loff_t *_next)
 {
struct folio *folio;
-   struct page *head_page;
+   struct folio_batch fbatch;
ssize_t ret;
+   unsigned int i;
int n, skips = 0;
 
_enter("%llx,%llx,", start, end);
+   folio_batch_init(&fbatch);
 
do {
pgoff_t index = start / PAGE_SIZE;
 
-   n = find_get_pages_range_tag(mapping, &index, end / PAGE_SIZE,
-PAGECACHE_TAG_DIRTY, 1, 
&head_page);
+   n = filemap_get_folios_tag(mapping, &index, end / PAGE_SIZE,
+   PAGECACHE_TAG_DIRTY, &fbatch);
+
if (!n)
break;
+   for (i = 0; i < n; i++) {
+   folio = fbatch.folios[i];
+   start = folio_pos(folio); /* May regress with THPs */
 
-   folio = page_folio(head_page);
-   start = folio_pos(folio); /* May regress with THPs */
-
-   _debug("wback %lx", folio_index(folio));
+   _debug("wback %lx", folio_index(folio));
 
-   /* At this point we hold neither the i_pages lock nor the
-* page lock: the page may be truncated or invalidated
-* (changing page->mapping to NULL), or even swizzled
-* back from swapper_space to tmpfs file mapping
-*/
-   if (wbc->sync_mode != WB_SYNC_NONE) {
-   ret = folio_lock_killable(folio);
-   if (ret < 0) {
-   folio_put(folio);
-   return ret;
-   }
-   } else {
-   if (!folio_trylock(folio)) {
-   folio_put(folio);
-   return 0;
+   /* At this point we hold neither the i_pages lock nor 
the
+* page lock: the page may be truncated or invalidated
+* (changing page->mapping to NULL), or even swizzled
+* back from swapper_space to tmpfs file mapping
+*/
+   if (wbc->sync_mode != WB_SYNC_NONE) {
+   ret = folio_lock_killable(folio);
+   if (ret < 0) {
+   folio_batch_release(&fbatch);
+   return ret;
+   }
+   } else {
+   if (!folio_trylock(folio))
+   continue;
}
-   }
 
-   if (folio_mapping(folio) != mapping ||
-   !folio_test_dirty(folio)) {
-   start += folio_size(folio);
-   folio_unlock(folio);
-   folio_put(folio);
-   continue;
-   }
+   if (folio->mapping != mapping ||
+   !folio_test_dirty(folio)) {
+   start += folio_size(folio);
+   folio_unlock(folio);
+   continue;
+   }
 
-   if (folio_test_writeback(folio) ||
-   folio_test_fscache(folio)) {
-   folio_unlock(folio);
-   if (wbc->sync_mode != WB_SYNC_NONE) {
-   folio_wait_writeback(folio);
+   if (folio_test_writeback(folio) ||
+   folio_test_fscache(folio)) {
+   folio_unlock(folio);
+   if (wbc->sync_mode != WB_SYNC_NONE) {
+   folio_wait_writeback(folio);
 #ifdef CONFIG_AFS_FSCACHE
-   folio_wait_fscache(folio);
+   folio_wait_fscache(folio);
 #endif
-   } else {
-   start += folio_size(folio);
+   } else {
+   start += folio_size(folio);
+   }
+   

[f2fs-dev] [PATCH v4 08/23] ceph: Convert ceph_writepages_start() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use a folio_batch instead of pagevec. This is in
preparation for the removal of find_get_pages_range_tag().

Also some minor renaming for consistency.

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Jeff Layton 
---
 fs/ceph/addr.c | 58 ++
 1 file changed, 30 insertions(+), 28 deletions(-)

diff --git a/fs/nilfs2/page.c b/fs/nilfs2/page.c
index d921542a9593..41ccd43cd979 100644
--- a/fs/nilfs2/page.c
+++ b/fs/nilfs2/page.c
@@ -358,22 +358,22 @@ void nilfs_copy_back_pages(struct address_space *dmap,
  */
 void nilfs_clear_dirty_pages(struct address_space *mapping, bool silent)
 {
-   struct pagevec pvec;
+   struct folio_batch fbatch;
unsigned int i;
pgoff_t index = 0;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
 
-   while (pagevec_lookup_tag(&pvec, mapping, &index,
-   PAGECACHE_TAG_DIRTY)) {
-   for (i = 0; i < pagevec_count(&pvec); i++) {
-   struct page *page = pvec.pages[i];
+   while (filemap_get_folios_tag(mapping, &index, (pgoff_t)-1,
+   PAGECACHE_TAG_DIRTY, &fbatch)) {
+   for (i = 0; i < folio_batch_count(&fbatch); i++) {
+   struct folio *folio = fbatch.folios[i];
 
-   lock_page(page);
-   nilfs_clear_dirty_page(page, silent);
-   unlock_page(page);
+   folio_lock(folio);
+   nilfs_clear_dirty_page(&folio->page, silent);
+   folio_unlock(folio);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
 }
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 13/23] f2fs: Convert f2fs_sync_node_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use a folio_batch instead of pagevec. This is in
preparation for the removal of find_get_pages_range_tag().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Chao Yu 
---
 fs/f2fs/node.c | 17 +
 1 file changed, 9 insertions(+), 8 deletions(-)

diff --git a/fs/f2fs/node.c b/fs/f2fs/node.c
index a2f477cc48c7..38f32b4d61dc 100644
--- a/fs/f2fs/node.c
+++ b/fs/f2fs/node.c
@@ -1935,23 +1935,24 @@ int f2fs_sync_node_pages(struct f2fs_sb_info *sbi,
bool do_balance, enum iostat_type io_type)
 {
pgoff_t index;
-   struct pagevec pvec;
+   struct folio_batch fbatch;
int step = 0;
int nwritten = 0;
int ret = 0;
-   int nr_pages, done = 0;
+   int nr_folios, done = 0;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
 
 next_step:
index = 0;
 
-   while (!done && (nr_pages = pagevec_lookup_tag(&pvec,
-   NODE_MAPPING(sbi), &index, PAGECACHE_TAG_DIRTY))) {
+   while (!done && (nr_folios = filemap_get_folios_tag(NODE_MAPPING(sbi),
+   &index, (pgoff_t)-1, PAGECACHE_TAG_DIRTY,
+   &fbatch))) {
int i;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct page *page = &fbatch.folios[i]->page;
bool submitted = false;
 
/* give a priority to WB_SYNC threads */
@@ -2026,7 +2027,7 @@ int f2fs_sync_node_pages(struct f2fs_sb_info *sbi,
if (--wbc->nr_to_write == 0)
break;
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
 
if (wbc->nr_to_write == 0) {
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 21/23] nilfs2: Convert nilfs_copy_dirty_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag(). This change removes 8 calls
to compound_head().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Ryusuke Konishi 
---
 fs/nilfs2/page.c | 39 ---
 1 file changed, 20 insertions(+), 19 deletions(-)

diff --git a/fs/nilfs2/page.c b/fs/nilfs2/page.c
index 39b7eea2642a..d921542a9593 100644
--- a/fs/nilfs2/page.c
+++ b/fs/nilfs2/page.c
@@ -240,42 +240,43 @@ static void nilfs_copy_page(struct page *dst, struct page 
*src, int copy_dirty)
 int nilfs_copy_dirty_pages(struct address_space *dmap,
   struct address_space *smap)
 {
-   struct pagevec pvec;
+   struct folio_batch fbatch;
unsigned int i;
pgoff_t index = 0;
int err = 0;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
 repeat:
-   if (!pagevec_lookup_tag(&pvec, smap, &index, PAGECACHE_TAG_DIRTY))
+   if (!filemap_get_folios_tag(smap, &index, (pgoff_t)-1,
+   PAGECACHE_TAG_DIRTY, &fbatch))
return 0;
 
-   for (i = 0; i < pagevec_count(&pvec); i++) {
-   struct page *page = pvec.pages[i], *dpage;
+   for (i = 0; i < folio_batch_count(&fbatch); i++) {
+   struct folio *folio = fbatch.folios[i], *dfolio;
 
-   lock_page(page);
-   if (unlikely(!PageDirty(page)))
-   NILFS_PAGE_BUG(page, "inconsistent dirty state");
+   folio_lock(folio);
+   if (unlikely(!folio_test_dirty(folio)))
+   NILFS_PAGE_BUG(&folio->page, "inconsistent dirty 
state");
 
-   dpage = grab_cache_page(dmap, page->index);
-   if (unlikely(!dpage)) {
+   dfolio = filemap_grab_folio(dmap, folio->index);
+   if (unlikely(!dfolio)) {
/* No empty page is added to the page cache */
err = -ENOMEM;
-   unlock_page(page);
+   folio_unlock(folio);
break;
}
-   if (unlikely(!page_has_buffers(page)))
-   NILFS_PAGE_BUG(page,
+   if (unlikely(!folio_buffers(folio)))
+   NILFS_PAGE_BUG(&folio->page,
   "found empty page in dat page cache");
 
-   nilfs_copy_page(dpage, page, 1);
-   __set_page_dirty_nobuffers(dpage);
+   nilfs_copy_page(&dfolio->page, &folio->page, 1);
+   filemap_dirty_folio(folio_mapping(dfolio), dfolio);
 
-   unlock_page(dpage);
-   put_page(dpage);
-   unlock_page(page);
+   folio_unlock(dfolio);
+   folio_put(dfolio);
+   folio_unlock(folio);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
 
if (likely(!err))
-- 

[f2fs-dev] [PATCH v4 19/23] nilfs2: Convert nilfs_lookup_dirty_node_buffers() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag(). This change removes 1 call to
compound_head().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Ryusuke Konishi 
---
 fs/nilfs2/segment.c | 15 +++
 1 file changed, 7 insertions(+), 8 deletions(-)

diff --git a/fs/nilfs2/segment.c b/fs/nilfs2/segment.c
index 2183e1698f8e..fe984def1b1c 100644
--- a/fs/nilfs2/segment.c
+++ b/fs/nilfs2/segment.c
@@ -737,20 +737,19 @@ static void nilfs_lookup_dirty_node_buffers(struct inode 
*inode,
 {
struct nilfs_inode_info *ii = NILFS_I(inode);
struct inode *btnc_inode = ii->i_assoc_inode;
-   struct pagevec pvec;
+   struct folio_batch fbatch;
struct buffer_head *bh, *head;
unsigned int i;
pgoff_t index = 0;
 
if (!btnc_inode)
return;
+   folio_batch_init(&fbatch);
 
-   pagevec_init(&pvec);
-
-   while (pagevec_lookup_tag(&pvec, btnc_inode->i_mapping, &index,
-   PAGECACHE_TAG_DIRTY)) {
-   for (i = 0; i < pagevec_count(&pvec); i++) {
-   bh = head = page_buffers(pvec.pages[i]);
+   while (filemap_get_folios_tag(btnc_inode->i_mapping, &index,
+   (pgoff_t)-1, PAGECACHE_TAG_DIRTY, &fbatch)) {
+   for (i = 0; i < folio_batch_count(&fbatch); i++) {
+   bh = head = folio_buffers(fbatch.folios[i]);
do {
if (buffer_dirty(bh) &&
!buffer_async_write(bh)) {
@@ -761,7 +760,7 @@ static void nilfs_lookup_dirty_node_buffers(struct inode 
*inode,
bh = bh->b_this_page;
} while (bh != head);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
 }
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [Bug 216050] f2fs_gc occupies 100% cpu

2022-11-02 Thread bugzilla-daemon
https://bugzilla.kernel.org/show_bug.cgi?id=216050

--- Comment #71 from Guido (guido.iod...@gmail.com) ---
Since the problem occurred with the 5.18 kernel, I assume there is something
wrong after that version. I think the analysis of the problem should start with
the changes that occurred with version 5.18.

At the moment, I am still stuck on version 5.17.9 for this very reason, which
in fact prevents me from using the PC with more recent versions of the kernel.

-- 
You may reply to this email to add a comment.

You are receiving this mail because:
You are watching the assignee of the bug.

___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


Re: [f2fs-dev] f2fs_empty_dir() can be extremely slow on malicious disk images

2022-11-02 Thread Chao Yu

On 2022/11/2 23:12, Chao Yu wrote:

On 2022/11/2 14:17, Eric Biggers wrote:

[+f2fs list and maintainers]


Thanks for the forwarding.


[changed subject from "INFO: task hung in fscrypt_ioctl_set_policy"]

On Mon, Oct 31, 2022 at 10:18:02PM +0800, Wei Chen wrote:

Dear Linux developers,

Here is the link to the reproducers.

C reproducer: 
https://drive.google.com/file/d/1mduYsYuoOKemH3qkvpDQwnAHAaaLUp0Y/view?usp=share_link
Syz reproducer:
https://drive.google.com/file/d/1mu-_w7dy_562vWRlQvTRbcBjG4_G7b2L/view?usp=share_link

The bug persists in the latest commit, v5.15.76 (4f5365f77018). I hope
it is helpful to you.

[ 1782.137186][   T30] INFO: task a.out:6910 blocked for more than 143 seconds.
[ 1782.139217][   T30]   Not tainted 5.15.76 #5
[ 1782.140388][   T30] "echo 0 >
/proc/sys/kernel/hung_task_timeout_secs" disables this message.
[ 1782.142524][   T30] task:a.out   state:D stack:14296 pid:
6910 ppid:  6532 flags:0x4004
[ 1782.144799][   T30] Call Trace:
[ 1782.145623][   T30]  
[ 1782.146316][   T30]  __schedule+0x3e8/0x1850
[ 1782.152029][   T30]  ? mark_held_locks+0x49/0x70
[ 1782.153533][   T30]  ? mark_held_locks+0x10/0x70
[ 1782.154759][   T30]  ? __down_write_common.part.14+0x31f/0x7b0
[ 1782.156159][   T30]  schedule+0x4e/0xe0
[ 1782.158314][   T30]  __down_write_common.part.14+0x324/0x7b0
[ 1782.159704][   T30]  ? fscrypt_ioctl_set_policy+0xe0/0x200
[ 1782.161050][   T30]  fscrypt_ioctl_set_policy+0xe0/0x200
[ 1782.162330][   T30]  __f2fs_ioctl+0x9d6/0x45e0
[ 1782.163417][   T30]  f2fs_ioctl+0x64/0x240
[ 1782.164404][   T30]  ? __f2fs_ioctl+0x45e0/0x45e0
[ 1782.165554][   T30]  __x64_sys_ioctl+0xb6/0x100
[ 1782.12][   T30]  do_syscall_64+0x34/0xb0
[ 1782.169947][   T30]  entry_SYSCALL_64_after_hwframe+0x61/0xcb


Well, the quality of this bug report has a lot to be desired (not on upstream
kernel, reproducer is full of totally irrelevant stuff, not sent to the mailing
list of the filesystem whose disk image is being fuzzed, etc.).  But what is
going on is that f2fs_empty_dir() doesn't consider the case of a directory with
an extremely large i_size on a malicious disk image.

Specifically, the reproducer mounts an f2fs image with a directory that has an
i_size of 14814520042850357248, then calls FS_IOC_SET_ENCRYPTION_POLICY on it.
That results in a call to f2fs_empty_dir() to check whether the directory is
empty.  f2fs_empty_dir() then iterates through all 3616826182336513 blocks the
directory allegedly contains to check whether any contain anything.  i_rwsem is
held during this, so anything else that tries to take it will hang.

I'll look into this more if needed, but Jaegeuk and Chao, do you happen to have
any ideas for how f2fs_empty_dir() should be fixed?  Is there an easy way to
just iterate through the blocks that are actually allocated?




Sorry, I mean:

From 07f662ca6bd2a0991961ea42932ce90f19e74624 Mon Sep 17 00:00:00 2001
From: Chao Yu 
Date: Wed, 2 Nov 2022 12:02:08 +0800
Subject: [RFC v2] f2fs: speed up f2fs_empty_dir()

Signed-off-by: Chao Yu 
---
 fs/f2fs/dir.c | 48 
 1 file changed, 28 insertions(+), 20 deletions(-)

diff --git a/fs/f2fs/dir.c b/fs/f2fs/dir.c
index 21960a899b6a..569f7304e3e6 100644
--- a/fs/f2fs/dir.c
+++ b/fs/f2fs/dir.c
@@ -956,38 +956,46 @@ void f2fs_delete_entry(struct f2fs_dir_entry *dentry, 
struct page *page,

 bool f2fs_empty_dir(struct inode *dir)
 {
-   unsigned long bidx;
struct page *dentry_page;
unsigned int bit_pos;
struct f2fs_dentry_block *dentry_blk;
-   unsigned long nblock = dir_blocks(dir);
+   pgoff_t index;

if (f2fs_has_inline_dentry(dir))
return f2fs_empty_inline_dir(dir);

-   for (bidx = 0; bidx < nblock; bidx++) {
-   dentry_page = f2fs_get_lock_data_page(dir, bidx, false);
-   if (IS_ERR(dentry_page)) {
-   if (PTR_ERR(dentry_page) == -ENOENT)
-   continue;
-   else
-   return false;
-   }
+   dentry_page = f2fs_get_lock_data_page(dir, 0, false);
+   if (IS_ERR(dentry_page)) {
+   if (PTR_ERR(dentry_page) == -ENOENT)
+   return true;
+   return false;
+   }

-   dentry_blk = page_address(dentry_page);
-   if (bidx == 0)
-   bit_pos = 2;
-   else
-   bit_pos = 0;
-   bit_pos = find_next_bit_le(&dentry_blk->dentry_bitmap,
-   NR_DENTRY_IN_BLOCK,
-   bit_pos);
+   dentry_blk = page_address(dentry_page);
+   bit_pos = find_next_bit_le(&dentry_blk->dentry_bitmap,
+   NR_DENTRY_IN_BLOCK, 2);
+   f2fs_put_page(dentry_page, 1);
+   
+   if (bit_pos < NR_DENTRY_IN_BLOCK)
+   ret

[f2fs-dev] [PATCH v1] f2fs: avoid victim selection from previous victim section

2022-11-02 Thread Yonggil Song
When f2fs chooses GC victim in large section & LFS mode,
next_victim_seg[gc_type] is referenced first. After segment is freed,
next_victim_seg[gc_type] has the next segment number.
However, next_victim_seg[gc_type] still has the last segment number
even after the last segment of section is freed. In this case, when f2fs
chooses a victim for the next GC round, the last segment of previous victim
section is chosen as a victim.

Initialize next_victim_seg[gc_type] to NULL_SEGNO for the last segment in
large section.

Signed-off-by: Yonggil Song 
---
 fs/f2fs/gc.c | 5 +++--
 1 file changed, 3 insertions(+), 2 deletions(-)

diff --git a/fs/f2fs/gc.c b/fs/f2fs/gc.c
index 4546e01b2ee0..10677d53ef0e 100644
--- a/fs/f2fs/gc.c
+++ b/fs/f2fs/gc.c
@@ -1744,8 +1744,9 @@ static int do_garbage_collect(struct f2fs_sb_info *sbi,
get_valid_blocks(sbi, segno, false) == 0)
seg_freed++;
 
-   if (__is_large_section(sbi) && segno + 1 < end_segno)
-   sbi->next_victim_seg[gc_type] = segno + 1;
+   if (__is_large_section(sbi))
+   sbi->next_victim_seg[gc_type] =
+   (segno + 1 < end_segno) ? segno + 1 : 
NULL_SEGNO;
 skip:
f2fs_put_page(sum_page, 0);
}
-- 
2.34.1


___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


Re: [f2fs-dev] [PATCH] f2fs: separate IPU policy for fdatasync from F2FS_IPU_FSYNC

2022-11-02 Thread qixiaoyu
Hi Chao,

fdatasync do in-place-update to avoid additional node writes, but currently
it only do that with F2FS_IPU_FSYNC as:

f2fs_do_sync_file:
if (datasync || get_dirty_pages(inode) <= SM_I(sbi)->min_fsync_blocks)
set_inode_flag(inode, FI_NEED_IPU);

check_inplace_update_policy:
/* this is only set during fdatasync */
if (policy & (0x1 << F2FS_IPU_FSYNC) &&
is_inode_flag_set(inode, FI_NEED_IPU))
return true;

So this patch separate in-place-update of fdatasync from F2FS_IPU_FSYNC to
apply it to all IPU policy.

BTW, we found small performance improvement with this patch on AndroBench app
using F2FS_IPU_SSR_UTIL on our product:

F2FS_IPU_FSYNC  F2FS_IPU_SSR_UTIL   F2FS_IPU_SSR_UTIL(with 
patch)
SQLite Insert(QPS)  6818.08 6327.09(-7.20%) 6757.72
SQLite Update(QPS)  6528.81 6336.57(-2.94%) 6490.77
SQLite Delete(QPS)  9724.68 9378.37(-3.56%) 9622.27

Thanks

On Tue, Nov 01, 2022 at 11:14:55PM +0800, Chao Yu wrote:
> On 2022/10/21 10:31, qixiaoyu1 wrote:
> >Currently IPU policy for fdatasync is coupled with F2FS_IPU_FSYNC.
> >Fix to apply it to all IPU policy.
> 
> Xiaoyu,
> 
> Sorry for the delay.
> 
> I didn't get the point, can you please explain more about the
> issue?
> 
> Thanks,
> 
> >
> >Signed-off-by: qixiaoyu1 
> >---
> >  fs/f2fs/data.c | 8 +++-
> >  fs/f2fs/file.c | 4 +++-
> >  2 files changed, 6 insertions(+), 6 deletions(-)
> >
> >diff --git a/fs/f2fs/data.c b/fs/f2fs/data.c
> >index a71e818cd67b..fec8e15fe820 100644
> >--- a/fs/f2fs/data.c
> >+++ b/fs/f2fs/data.c
> >@@ -2518,6 +2518,9 @@ static inline bool check_inplace_update_policy(struct 
> >inode *inode,
> > if (policy & (0x1 << F2FS_IPU_HONOR_OPU_WRITE) &&
> > is_inode_flag_set(inode, FI_OPU_WRITE))
> > return false;
> >+/* this is set by fdatasync or F2FS_IPU_FSYNC policy */
> >+if (is_inode_flag_set(inode, FI_NEED_IPU))
> >+return true;
> > if (policy & (0x1 << F2FS_IPU_FORCE))
> > return true;
> > if (policy & (0x1 << F2FS_IPU_SSR) && f2fs_need_SSR(sbi))
> >@@ -2538,11 +2541,6 @@ static inline bool check_inplace_update_policy(struct 
> >inode *inode,
> > !IS_ENCRYPTED(inode))
> > return true;
> >-/* this is only set during fdatasync */
> >-if (policy & (0x1 << F2FS_IPU_FSYNC) &&
> >-is_inode_flag_set(inode, FI_NEED_IPU))
> >-return true;
> >-
> > if (unlikely(fio && is_sbi_flag_set(sbi, SBI_CP_DISABLED) &&
> > !f2fs_is_checkpointed_data(sbi, fio->old_blkaddr)))
> > return true;
> >diff --git a/fs/f2fs/file.c b/fs/f2fs/file.c
> >index 82cda1258227..08091550cdf2 100644
> >--- a/fs/f2fs/file.c
> >+++ b/fs/f2fs/file.c
> >@@ -270,8 +270,10 @@ static int f2fs_do_sync_file(struct file *file, loff_t 
> >start, loff_t end,
> > goto go_write;
> > /* if fdatasync is triggered, let's do in-place-update */
> >-if (datasync || get_dirty_pages(inode) <= SM_I(sbi)->min_fsync_blocks)
> >+if (datasync || (SM_I(sbi)->ipu_policy & (0x1 << F2FS_IPU_FSYNC) &&
> >+get_dirty_pages(inode) <= SM_I(sbi)->min_fsync_blocks))
> > set_inode_flag(inode, FI_NEED_IPU);
> >+
> > ret = file_write_and_wait_range(file, start, end);
> > clear_inode_flag(inode, FI_NEED_IPU);


___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


Re: [f2fs-dev] f2fs_empty_dir() can be extremely slow on malicious disk images

2022-11-02 Thread Chao Yu

On 2022/11/2 14:17, Eric Biggers wrote:

[+f2fs list and maintainers]


Thanks for the forwarding.


[changed subject from "INFO: task hung in fscrypt_ioctl_set_policy"]

On Mon, Oct 31, 2022 at 10:18:02PM +0800, Wei Chen wrote:

Dear Linux developers,

Here is the link to the reproducers.

C reproducer: 
https://drive.google.com/file/d/1mduYsYuoOKemH3qkvpDQwnAHAaaLUp0Y/view?usp=share_link
Syz reproducer:
https://drive.google.com/file/d/1mu-_w7dy_562vWRlQvTRbcBjG4_G7b2L/view?usp=share_link

The bug persists in the latest commit, v5.15.76 (4f5365f77018). I hope
it is helpful to you.

[ 1782.137186][   T30] INFO: task a.out:6910 blocked for more than 143 seconds.
[ 1782.139217][   T30]   Not tainted 5.15.76 #5
[ 1782.140388][   T30] "echo 0 >
/proc/sys/kernel/hung_task_timeout_secs" disables this message.
[ 1782.142524][   T30] task:a.out   state:D stack:14296 pid:
6910 ppid:  6532 flags:0x4004
[ 1782.144799][   T30] Call Trace:
[ 1782.145623][   T30]  
[ 1782.146316][   T30]  __schedule+0x3e8/0x1850
[ 1782.152029][   T30]  ? mark_held_locks+0x49/0x70
[ 1782.153533][   T30]  ? mark_held_locks+0x10/0x70
[ 1782.154759][   T30]  ? __down_write_common.part.14+0x31f/0x7b0
[ 1782.156159][   T30]  schedule+0x4e/0xe0
[ 1782.158314][   T30]  __down_write_common.part.14+0x324/0x7b0
[ 1782.159704][   T30]  ? fscrypt_ioctl_set_policy+0xe0/0x200
[ 1782.161050][   T30]  fscrypt_ioctl_set_policy+0xe0/0x200
[ 1782.162330][   T30]  __f2fs_ioctl+0x9d6/0x45e0
[ 1782.163417][   T30]  f2fs_ioctl+0x64/0x240
[ 1782.164404][   T30]  ? __f2fs_ioctl+0x45e0/0x45e0
[ 1782.165554][   T30]  __x64_sys_ioctl+0xb6/0x100
[ 1782.12][   T30]  do_syscall_64+0x34/0xb0
[ 1782.169947][   T30]  entry_SYSCALL_64_after_hwframe+0x61/0xcb


Well, the quality of this bug report has a lot to be desired (not on upstream
kernel, reproducer is full of totally irrelevant stuff, not sent to the mailing
list of the filesystem whose disk image is being fuzzed, etc.).  But what is
going on is that f2fs_empty_dir() doesn't consider the case of a directory with
an extremely large i_size on a malicious disk image.

Specifically, the reproducer mounts an f2fs image with a directory that has an
i_size of 14814520042850357248, then calls FS_IOC_SET_ENCRYPTION_POLICY on it.
That results in a call to f2fs_empty_dir() to check whether the directory is
empty.  f2fs_empty_dir() then iterates through all 3616826182336513 blocks the
directory allegedly contains to check whether any contain anything.  i_rwsem is
held during this, so anything else that tries to take it will hang.

I'll look into this more if needed, but Jaegeuk and Chao, do you happen to have
any ideas for how f2fs_empty_dir() should be fixed?  Is there an easy way to
just iterate through the blocks that are actually allocated?


I send this just for requesting comments, no test now.

Thoughts?

From 38ea5f172c47853536a9f70857e4438a69d16f39 Mon Sep 17 00:00:00 2001
From: Chao Yu 
Date: Wed, 2 Nov 2022 12:02:08 +0800
Subject: [RFC PATCH] f2fs: speed up f2fs_empty_dir()

Signed-off-by: Chao Yu 
---
 fs/f2fs/dir.c | 48 ++--
 1 file changed, 26 insertions(+), 22 deletions(-)

diff --git a/fs/f2fs/dir.c b/fs/f2fs/dir.c
index 21960a899b6a..45f52b34ed1f 100644
--- a/fs/f2fs/dir.c
+++ b/fs/f2fs/dir.c
@@ -956,38 +956,42 @@ void f2fs_delete_entry(struct f2fs_dir_entry *dentry, 
struct page *page,

 bool f2fs_empty_dir(struct inode *dir)
 {
-   unsigned long bidx;
struct page *dentry_page;
unsigned int bit_pos;
struct f2fs_dentry_block *dentry_blk;
-   unsigned long nblock = dir_blocks(dir);
+   struct f2fs_map_blocks map;
+   int ret;

if (f2fs_has_inline_dentry(dir))
return f2fs_empty_inline_dir(dir);

-   for (bidx = 0; bidx < nblock; bidx++) {
-   dentry_page = f2fs_get_lock_data_page(dir, bidx, false);
-   if (IS_ERR(dentry_page)) {
-   if (PTR_ERR(dentry_page) == -ENOENT)
-   continue;
-   else
-   return false;
-   }
+   dentry_page = f2fs_get_lock_data_page(dir, 0, false);
+   if (IS_ERR(dentry_page)) {
+   if (PTR_ERR(dentry_page) == -ENOENT)
+   return true;
+   return false;
+   }

-   dentry_blk = page_address(dentry_page);
-   if (bidx == 0)
-   bit_pos = 2;
-   else
-   bit_pos = 0;
-   bit_pos = find_next_bit_le(&dentry_blk->dentry_bitmap,
-   NR_DENTRY_IN_BLOCK,
-   bit_pos);
+   dentry_blk = page_address(dentry_page);
+   bit_pos = find_next_bit_le(&dentry_blk->dentry_bitmap,
+   NR_DENTRY_IN_BLOCK, 2);
+   f2fs_put_page(dentry_page, 1);
+   
+   if (bit

[f2fs-dev] [PATCH v4 06/23] btrfs: Convert btree_write_cache_pages() to use filemap_get_folio_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Converted function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: David Sterba 
---
 fs/btrfs/extent_io.c | 19 ++-
 1 file changed, 10 insertions(+), 9 deletions(-)

diff --git a/fs/btrfs/extent_io.c b/fs/btrfs/extent_io.c
index 4dcf22e051ff..9ae75db4d55e 100644
--- a/fs/btrfs/extent_io.c
+++ b/fs/btrfs/extent_io.c
@@ -2960,14 +2960,14 @@ int btree_write_cache_pages(struct address_space 
*mapping,
int ret = 0;
int done = 0;
int nr_to_write_done = 0;
-   struct pagevec pvec;
-   int nr_pages;
+   struct folio_batch fbatch;
+   unsigned int nr_folios;
pgoff_t index;
pgoff_t end;/* Inclusive */
int scanned = 0;
xa_mark_t tag;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
if (wbc->range_cyclic) {
index = mapping->writeback_index; /* Start from prev offset */
end = -1;
@@ -2990,14 +2990,15 @@ int btree_write_cache_pages(struct address_space 
*mapping,
if (wbc->sync_mode == WB_SYNC_ALL)
tag_pages_for_writeback(mapping, index, end);
while (!done && !nr_to_write_done && (index <= end) &&
-  (nr_pages = pagevec_lookup_range_tag(&pvec, mapping, &index, end,
-   tag))) {
+  (nr_folios = filemap_get_folios_tag(mapping, &index, end,
+   tag, &fbatch))) {
unsigned i;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch.folios[i];
 
-   ret = submit_eb_page(page, wbc, &epd, &eb_context);
+   ret = submit_eb_page(&folio->page, wbc, &epd,
+   &eb_context);
if (ret == 0)
continue;
if (ret < 0) {
@@ -3012,7 +3013,7 @@ int btree_write_cache_pages(struct address_space *mapping,
 */
nr_to_write_done = wbc->nr_to_write <= 0;
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
if (!scanned && !done) {
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 07/23] btrfs: Convert extent_write_cache_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Converted function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag(). Now also supports large
folios.

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: David Sterba 
---
 fs/btrfs/extent_io.c | 38 +++---
 1 file changed, 19 insertions(+), 19 deletions(-)

diff --git a/fs/btrfs/extent_io.c b/fs/btrfs/extent_io.c
index 9ae75db4d55e..983dde83ba93 100644
--- a/fs/btrfs/extent_io.c
+++ b/fs/btrfs/extent_io.c
@@ -3088,8 +3088,8 @@ static int extent_write_cache_pages(struct address_space 
*mapping,
int ret = 0;
int done = 0;
int nr_to_write_done = 0;
-   struct pagevec pvec;
-   int nr_pages;
+   struct folio_batch fbatch;
+   unsigned int nr_folios;
pgoff_t index;
pgoff_t end;/* Inclusive */
pgoff_t done_index;
@@ -3109,7 +3109,7 @@ static int extent_write_cache_pages(struct address_space 
*mapping,
if (!igrab(inode))
return 0;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
if (wbc->range_cyclic) {
index = mapping->writeback_index; /* Start from prev offset */
end = -1;
@@ -3147,14 +3147,14 @@ static int extent_write_cache_pages(struct 
address_space *mapping,
tag_pages_for_writeback(mapping, index, end);
done_index = index;
while (!done && !nr_to_write_done && (index <= end) &&
-   (nr_pages = pagevec_lookup_range_tag(&pvec, mapping,
-   &index, end, tag))) {
+   (nr_folios = filemap_get_folios_tag(mapping, &index,
+   end, tag, &fbatch))) {
unsigned i;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch.folios[i];
 
-   done_index = page->index + 1;
+   done_index = folio->index + folio_nr_pages(folio);
/*
 * At this point we hold neither the i_pages lock nor
 * the page lock: the page may be truncated or
@@ -3162,29 +3162,29 @@ static int extent_write_cache_pages(struct 
address_space *mapping,
 * or even swizzled back from swapper_space to
 * tmpfs file mapping
 */
-   if (!trylock_page(page)) {
+   if (!folio_trylock(folio)) {
submit_write_bio(epd, 0);
-   lock_page(page);
+   folio_lock(folio);
}
 
-   if (unlikely(page->mapping != mapping)) {
-   unlock_page(page);
+   if (unlikely(folio->mapping != mapping)) {
+   folio_unlock(folio);
continue;
}
 
if (wbc->sync_mode != WB_SYNC_NONE) {
-   if (PageWriteback(page))
+   if (folio_test_writeback(folio))
submit_write_bio(epd, 0);
-   wait_on_page_writeback(page);
+   folio_wait_writeback(folio);
}
 
-   if (PageWriteback(page) ||
-   !clear_page_dirty_for_io(page)) {
-   unlock_page(page);
+   if (folio_test_writeback(folio) ||
+   !folio_clear_dirty_for_io(folio)) {
+   folio_unlock(folio);
continue;
}
 
-   ret = __extent_writepage(page, wbc, epd);
+   ret = __extent_writepage(&folio->page, wbc, epd);
if (ret < 0) {
done = 1;
break;
@@ -3197,7 +3197,7 @@ static int extent_write_cache_pages(struct address_space 
*mapping,
 */
nr_to_write_done = wbc->nr_to_write <= 0;
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
if (!scanned && !done) {
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 00/23] Convert to filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
This patch series replaces find_get_pages_range_tag() with
filemap_get_folios_tag(). This also allows the removal of multiple
calls to compound_head() throughout.
It also makes a good chunk of the straightforward conversions to folios,
and takes the opportunity to introduce a function that grabs a folio
from the pagecache.

F2fs and Ceph have quite a lot of work to be done regarding folios, so
for now those patches only have the changes necessary for the removal of
find_get_pages_range_tag(), and only support folios of size 1 (which is
all they use right now anyways).

I've run xfstests on btrfs, ext4, f2fs, and nilfs2, but more testing may be
beneficial. The page-writeback and filemap changes implicitly work. Testing
and review of the other changes (afs, ceph, cifs, gfs2) would be appreciated.

---
v4:
  Fixed a bug with reference counting in cifs changes
  - Reported-by: kernel test robot  
  Improved commit messages to be more meaningful
  Got some Acked-bys and Reviewed-bys

v3:
  Rebased onto upstream 6.1
  Simplified the ceph patch to only necessary changes
  Changed commit messages throughout to be clearer
  Got an Acked-by for another nilfs patch
  Got Tested-by for afs

v2:
  Got Acked-By tags for nilfs and btrfs changes
  Fixed an error arising in f2fs
  - Reported-by: kernel test robot 

Vishal Moola (Oracle) (23):
  pagemap: Add filemap_grab_folio()
  filemap: Added filemap_get_folios_tag()
  filemap: Convert __filemap_fdatawait_range() to use
filemap_get_folios_tag()
  page-writeback: Convert write_cache_pages() to use
filemap_get_folios_tag()
  afs: Convert afs_writepages_region() to use filemap_get_folios_tag()
  btrfs: Convert btree_write_cache_pages() to use
filemap_get_folio_tag()
  btrfs: Convert extent_write_cache_pages() to use
filemap_get_folios_tag()
  ceph: Convert ceph_writepages_start() to use filemap_get_folios_tag()
  cifs: Convert wdata_alloc_and_fillpages() to use
filemap_get_folios_tag()
  ext4: Convert mpage_prepare_extent_to_map() to use
filemap_get_folios_tag()
  f2fs: Convert f2fs_fsync_node_pages() to use filemap_get_folios_tag()
  f2fs: Convert f2fs_flush_inline_data() to use filemap_get_folios_tag()
  f2fs: Convert f2fs_sync_node_pages() to use filemap_get_folios_tag()
  f2fs: Convert f2fs_write_cache_pages() to use filemap_get_folios_tag()
  f2fs: Convert last_fsync_dnode() to use filemap_get_folios_tag()
  f2fs: Convert f2fs_sync_meta_pages() to use filemap_get_folios_tag()
  gfs2: Convert gfs2_write_cache_jdata() to use filemap_get_folios_tag()
  nilfs2: Convert nilfs_lookup_dirty_data_buffers() to use
filemap_get_folios_tag()
  nilfs2: Convert nilfs_lookup_dirty_node_buffers() to use
filemap_get_folios_tag()
  nilfs2: Convert nilfs_btree_lookup_dirty_buffers() to use
filemap_get_folios_tag()
  nilfs2: Convert nilfs_copy_dirty_pages() to use
filemap_get_folios_tag()
  nilfs2: Convert nilfs_clear_dirty_pages() to use
filemap_get_folios_tag()
  filemap: Remove find_get_pages_range_tag()

 fs/afs/write.c  | 114 +---
 fs/btrfs/extent_io.c|  57 ++--
 fs/ceph/addr.c  |  58 ++--
 fs/cifs/file.c  |  32 +--
 fs/ext4/inode.c |  55 ++-
 fs/f2fs/checkpoint.c|  49 +
 fs/f2fs/compress.c  |  13 ++---
 fs/f2fs/data.c  |  69 +---
 fs/f2fs/f2fs.h  |   5 +-
 fs/f2fs/node.c  |  72 +
 fs/gfs2/aops.c  |  64 --
 fs/nilfs2/btree.c   |  14 ++---
 fs/nilfs2/page.c|  59 +++--
 fs/nilfs2/segment.c |  44 
 include/linux/pagemap.h |  32 +++
 include/linux/pagevec.h |   8 ---
 mm/filemap.c|  84 ++---
 mm/page-writeback.c |  44 
 mm/swap.c   |  10 
 19 files changed, 465 insertions(+), 418 deletions(-)

-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 03/23] filemap: Convert __filemap_fdatawait_range() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Converted function to use folios. This is in preparation for the removal
of find_get_pages_range_tag(). This change removes 2 calls to
compound_head().

Signed-off-by: Vishal Moola (Oracle) 
Reviewed-by: Matthew Wilcow (Oracle) 
---
 mm/filemap.c | 24 +---
 1 file changed, 13 insertions(+), 11 deletions(-)

diff --git a/mm/filemap.c b/mm/filemap.c
index 9be22672ce1a..cc4be51eae5b 100644
--- a/mm/filemap.c
+++ b/mm/filemap.c
@@ -503,28 +503,30 @@ static void __filemap_fdatawait_range(struct 
address_space *mapping,
 {
pgoff_t index = start_byte >> PAGE_SHIFT;
pgoff_t end = end_byte >> PAGE_SHIFT;
-   struct pagevec pvec;
-   int nr_pages;
+   struct folio_batch fbatch;
+   unsigned nr_folios;
 
if (end_byte < start_byte)
return;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
+
while (index <= end) {
unsigned i;
 
-   nr_pages = pagevec_lookup_range_tag(&pvec, mapping, &index,
-   end, PAGECACHE_TAG_WRITEBACK);
-   if (!nr_pages)
+   nr_folios = filemap_get_folios_tag(mapping, &index, end,
+   PAGECACHE_TAG_WRITEBACK, &fbatch);
+
+   if (!nr_folios)
break;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch.folios[i];
 
-   wait_on_page_writeback(page);
-   ClearPageError(page);
+   folio_wait_writeback(folio);
+   folio_clear_error(folio);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
 }
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 02/23] filemap: Added filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
This is the equivalent of find_get_pages_range_tag(), except for folios
instead of pages.

One noteable difference is filemap_get_folios_tag() does not take in a
maximum pages argument. It instead tries to fill a folio batch and stops
either once full (15 folios) or reaching the end of the search range.

The new function supports large folios, the initial function did not
since all callers don't use large folios.

Signed-off-by: Vishal Moola (Oracle) 
Reviewed-by: Matthew Wilcow (Oracle) 
---
 include/linux/pagemap.h |  2 ++
 mm/filemap.c| 54 +
 2 files changed, 56 insertions(+)

diff --git a/include/linux/pagemap.h b/include/linux/pagemap.h
index 74d87e37a142..28275eecb949 100644
--- a/include/linux/pagemap.h
+++ b/include/linux/pagemap.h
@@ -740,6 +740,8 @@ unsigned filemap_get_folios(struct address_space *mapping, 
pgoff_t *start,
pgoff_t end, struct folio_batch *fbatch);
 unsigned filemap_get_folios_contig(struct address_space *mapping,
pgoff_t *start, pgoff_t end, struct folio_batch *fbatch);
+unsigned filemap_get_folios_tag(struct address_space *mapping, pgoff_t *start,
+   pgoff_t end, xa_mark_t tag, struct folio_batch *fbatch);
 unsigned find_get_pages_range_tag(struct address_space *mapping, pgoff_t 
*index,
pgoff_t end, xa_mark_t tag, unsigned int nr_pages,
struct page **pages);
diff --git a/mm/filemap.c b/mm/filemap.c
index 08341616ae7a..9be22672ce1a 100644
--- a/mm/filemap.c
+++ b/mm/filemap.c
@@ -2262,6 +2262,60 @@ unsigned filemap_get_folios_contig(struct address_space 
*mapping,
 }
 EXPORT_SYMBOL(filemap_get_folios_contig);
 
+/**
+ * filemap_get_folios_tag - Get a batch of folios matching @tag
+ * @mapping:The address_space to search
+ * @start:  The starting page index
+ * @end:The final page index (inclusive)
+ * @tag:The tag index
+ * @fbatch: The batch to fill
+ *
+ * Same as filemap_get_folios(), but only returning folios tagged with @tag.
+ *
+ * Return: The number of folios found.
+ * Also update @start to index the next folio for traversal.
+ */
+unsigned filemap_get_folios_tag(struct address_space *mapping, pgoff_t *start,
+   pgoff_t end, xa_mark_t tag, struct folio_batch *fbatch)
+{
+   XA_STATE(xas, &mapping->i_pages, *start);
+   struct folio *folio;
+
+   rcu_read_lock();
+   while ((folio = find_get_entry(&xas, end, tag)) != NULL) {
+   /*
+* Shadow entries should never be tagged, but this iteration
+* is lockless so there is a window for page reclaim to evict
+* a page we saw tagged. Skip over it.
+*/
+   if (xa_is_value(folio))
+   continue;
+   if (!folio_batch_add(fbatch, folio)) {
+   unsigned long nr = folio_nr_pages(folio);
+
+   if (folio_test_hugetlb(folio))
+   nr = 1;
+   *start = folio->index + nr;
+   goto out;
+   }
+   }
+   /*
+* We come here when there is no page beyond @end. We take care to not
+* overflow the index @start as it confuses some of the callers. This
+* breaks the iteration when there is a page at index -1 but that is
+* already broke anyway.
+*/
+   if (end == (pgoff_t)-1)
+   *start = (pgoff_t)-1;
+   else
+   *start = end + 1;
+out:
+   rcu_read_unlock();
+
+   return folio_batch_count(fbatch);
+}
+EXPORT_SYMBOL(filemap_get_folios_tag);
+
 /**
  * find_get_pages_range_tag - Find and return head pages matching @tag.
  * @mapping:   the address_space to search
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 01/23] pagemap: Add filemap_grab_folio()

2022-11-02 Thread Vishal Moola (Oracle)
Add function filemap_grab_folio() to grab a folio from the page cache.
This function is meant to serve as a folio replacement for
grab_cache_page, and is used to facilitate the removal of
find_get_pages_range_tag().

Signed-off-by: Vishal Moola (Oracle) 
Reviewed-by: Matthew Wilcox (Oracle) 
---
 include/linux/pagemap.h | 20 
 1 file changed, 20 insertions(+)

diff --git a/include/linux/pagemap.h b/include/linux/pagemap.h
index bbccb4044222..74d87e37a142 100644
--- a/include/linux/pagemap.h
+++ b/include/linux/pagemap.h
@@ -547,6 +547,26 @@ static inline struct folio *filemap_lock_folio(struct 
address_space *mapping,
return __filemap_get_folio(mapping, index, FGP_LOCK, 0);
 }
 
+/**
+ * filemap_grab_folio - grab a folio from the page cache
+ * @mapping: The address space to search
+ * @index: The page index
+ *
+ * Looks up the page cache entry at @mapping & @index. If no folio is found,
+ * a new folio is created. The folio is locked, marked as accessed, and
+ * returned.
+ *
+ * Return: A found or created folio. NULL if no folio is found and failed to
+ * create a folio.
+ */
+static inline struct folio *filemap_grab_folio(struct address_space *mapping,
+   pgoff_t index)
+{
+   return __filemap_get_folio(mapping, index,
+   FGP_LOCK | FGP_ACCESSED | FGP_CREAT,
+   mapping_gfp_mask(mapping));
+}
+
 /**
  * find_get_page - find and get a page reference
  * @mapping: the address_space to search
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 10/23] ext4: Convert mpage_prepare_extent_to_map() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Converted the function to use folios throughout. This is in preparation
for the removal of find_get_pages_range_tag(). Now supports large
folios. This change removes 10 calls to compound_head().

Signed-off-by: Vishal Moola (Oracle) 
---
 fs/ext4/inode.c | 55 -
 1 file changed, 27 insertions(+), 28 deletions(-)

diff --git a/fs/ext4/inode.c b/fs/ext4/inode.c
index 2b5ef1b64249..69a0708c8e87 100644
--- a/fs/ext4/inode.c
+++ b/fs/ext4/inode.c
@@ -2572,8 +2572,8 @@ static int ext4_da_writepages_trans_blocks(struct inode 
*inode)
 static int mpage_prepare_extent_to_map(struct mpage_da_data *mpd)
 {
struct address_space *mapping = mpd->inode->i_mapping;
-   struct pagevec pvec;
-   unsigned int nr_pages;
+   struct folio_batch fbatch;
+   unsigned int nr_folios;
long left = mpd->wbc->nr_to_write;
pgoff_t index = mpd->first_page;
pgoff_t end = mpd->last_page;
@@ -2587,18 +2587,17 @@ static int mpage_prepare_extent_to_map(struct 
mpage_da_data *mpd)
tag = PAGECACHE_TAG_TOWRITE;
else
tag = PAGECACHE_TAG_DIRTY;
-
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
mpd->map.m_len = 0;
mpd->next_page = index;
while (index <= end) {
-   nr_pages = pagevec_lookup_range_tag(&pvec, mapping, &index, end,
-   tag);
-   if (nr_pages == 0)
+   nr_folios = filemap_get_folios_tag(mapping, &index, end,
+   tag, &fbatch);
+   if (nr_folios == 0)
break;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch.folios[i];
 
/*
 * Accumulated enough dirty pages? This doesn't apply
@@ -2612,10 +2611,10 @@ static int mpage_prepare_extent_to_map(struct 
mpage_da_data *mpd)
goto out;
 
/* If we can't merge this page, we are done. */
-   if (mpd->map.m_len > 0 && mpd->next_page != page->index)
+   if (mpd->map.m_len > 0 && mpd->next_page != 
folio->index)
goto out;
 
-   lock_page(page);
+   folio_lock(folio);
/*
 * If the page is no longer dirty, or its mapping no
 * longer corresponds to inode we are writing (which
@@ -2623,16 +2622,16 @@ static int mpage_prepare_extent_to_map(struct 
mpage_da_data *mpd)
 * page is already under writeback and we are not doing
 * a data integrity writeback, skip the page
 */
-   if (!PageDirty(page) ||
-   (PageWriteback(page) &&
+   if (!folio_test_dirty(folio) ||
+   (folio_test_writeback(folio) &&
 (mpd->wbc->sync_mode == WB_SYNC_NONE)) ||
-   unlikely(page->mapping != mapping)) {
-   unlock_page(page);
+   unlikely(folio->mapping != mapping)) {
+   folio_unlock(folio);
continue;
}
 
-   wait_on_page_writeback(page);
-   BUG_ON(PageWriteback(page));
+   folio_wait_writeback(folio);
+   BUG_ON(folio_test_writeback(folio));
 
/*
 * Should never happen but for buggy code in
@@ -2643,33 +2642,33 @@ static int mpage_prepare_extent_to_map(struct 
mpage_da_data *mpd)
 *
 * [1] 
https://lore.kernel.org/linux-mm/20180103100430.ge4...@quack2.suse.cz
 */
-   if (!page_has_buffers(page)) {
-   ext4_warning_inode(mpd->inode, "page %lu does 
not have buffers attached", page->index);
-   ClearPageDirty(page);
-   unlock_page(page);
+   if (!folio_buffers(folio)) {
+   ext4_warning_inode(mpd->inode, "page %lu does 
not have buffers attached", folio->index);
+   folio_clear_dirty(folio);
+   folio_unlock(folio);
continue;
}
 
if (mpd->map.m_len == 0)
-   mpd->first_page = page->index;
-   mpd->next_page = page->index + 1;
+   mpd->first_page = folio->index;
+   mpd->next_pa

[f2fs-dev] [PATCH v4 09/23] cifs: Convert wdata_alloc_and_fillpages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
This is in preparation for the removal of find_get_pages_range_tag(). Now also
supports the use of large folios.

Since tofind might be larger than the max number of folios in a
folio_batch (15), we loop through filling in wdata->pages pulling more
batches until we either reach tofind pages or run out of folios.

This function may not return all pages in the last found folio before
tofind pages are reached.

Signed-off-by: Vishal Moola (Oracle) 
---
 fs/cifs/file.c | 32 +---
 1 file changed, 29 insertions(+), 3 deletions(-)

diff --git a/fs/cifs/file.c b/fs/cifs/file.c
index f6ffee514c34..789fd0397cb4 100644
--- a/fs/cifs/file.c
+++ b/fs/cifs/file.c
@@ -2520,14 +2520,40 @@ wdata_alloc_and_fillpages(pgoff_t tofind, struct 
address_space *mapping,
  unsigned int *found_pages)
 {
struct cifs_writedata *wdata;
-
+   struct folio_batch fbatch;
+   unsigned int i, idx, p, nr;
wdata = cifs_writedata_alloc((unsigned int)tofind,
 cifs_writev_complete);
if (!wdata)
return NULL;
 
-   *found_pages = find_get_pages_range_tag(mapping, index, end,
-   PAGECACHE_TAG_DIRTY, tofind, wdata->pages);
+   folio_batch_init(&fbatch);
+   *found_pages = 0;
+
+again:
+   nr = filemap_get_folios_tag(mapping, index, end,
+   PAGECACHE_TAG_DIRTY, &fbatch);
+   if (!nr)
+   goto out; /* No dirty pages left in the range */
+
+   for (i = 0; i < nr; i++) {
+   struct folio *folio = fbatch.folios[i];
+
+   idx = 0;
+   p = folio_nr_pages(folio);
+add_more:
+   wdata->pages[*found_pages] = folio_page(folio, idx);
+   folio_ref_inc(folio);
+   if (++*found_pages == tofind) {
+   folio_batch_release(&fbatch);
+   goto out;
+   }
+   if (++idx < p)
+   goto add_more;
+   }
+   folio_batch_release(&fbatch);
+   goto again;
+out:
return wdata;
 }
 
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 14/23] f2fs: Convert f2fs_write_cache_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Converted the function to use a folio_batch instead of pagevec. This is in
preparation for the removal of find_get_pages_range_tag(). This change
removes 7 calls to compound_head().

Also modified f2fs_all_cluster_page_ready to take in a folio_batch instead
of pagevec. This does NOT support large folios. The function currently
only utilizes folios of size 1 so this shouldn't cause any issues right
now.

Signed-off-by: Vishal Moola (Oracle) 
---
 fs/f2fs/compress.c | 13 +
 fs/f2fs/data.c | 69 +-
 fs/f2fs/f2fs.h |  5 ++--
 3 files changed, 47 insertions(+), 40 deletions(-)

diff --git a/fs/f2fs/compress.c b/fs/f2fs/compress.c
index d315c2de136f..7af6c923e0aa 100644
--- a/fs/f2fs/compress.c
+++ b/fs/f2fs/compress.c
@@ -842,10 +842,11 @@ bool f2fs_cluster_can_merge_page(struct compress_ctx *cc, 
pgoff_t index)
return is_page_in_cluster(cc, index);
 }
 
-bool f2fs_all_cluster_page_ready(struct compress_ctx *cc, struct page **pages,
-   int index, int nr_pages, bool uptodate)
+bool f2fs_all_cluster_page_ready(struct compress_ctx *cc,
+   struct folio_batch *fbatch,
+   int index, int nr_folios, bool uptodate)
 {
-   unsigned long pgidx = pages[index]->index;
+   unsigned long pgidx = fbatch->folios[index]->index;
int i = uptodate ? 0 : 1;
 
/*
@@ -855,13 +856,13 @@ bool f2fs_all_cluster_page_ready(struct compress_ctx *cc, 
struct page **pages,
if (uptodate && (pgidx % cc->cluster_size))
return false;
 
-   if (nr_pages - index < cc->cluster_size)
+   if (nr_folios - index < cc->cluster_size)
return false;
 
for (; i < cc->cluster_size; i++) {
-   if (pages[index + i]->index != pgidx + i)
+   if (fbatch->folios[index + i]->index != pgidx + i)
return false;
-   if (uptodate && !PageUptodate(pages[index + i]))
+   if (uptodate && !folio_test_uptodate(fbatch->folios[index + i]))
return false;
}
 
diff --git a/fs/f2fs/data.c b/fs/f2fs/data.c
index a71e818cd67b..7511578b73c3 100644
--- a/fs/f2fs/data.c
+++ b/fs/f2fs/data.c
@@ -2938,7 +2938,7 @@ static int f2fs_write_cache_pages(struct address_space 
*mapping,
 {
int ret = 0;
int done = 0, retry = 0;
-   struct page *pages[F2FS_ONSTACK_PAGES];
+   struct folio_batch fbatch;
struct f2fs_sb_info *sbi = F2FS_M_SB(mapping);
struct bio *bio = NULL;
sector_t last_block;
@@ -2959,7 +2959,7 @@ static int f2fs_write_cache_pages(struct address_space 
*mapping,
.private = NULL,
};
 #endif
-   int nr_pages;
+   int nr_folios;
pgoff_t index;
pgoff_t end;/* Inclusive */
pgoff_t done_index;
@@ -2969,6 +2969,8 @@ static int f2fs_write_cache_pages(struct address_space 
*mapping,
int submitted = 0;
int i;
 
+   folio_batch_init(&fbatch);
+
if (get_dirty_pages(mapping->host) <=
SM_I(F2FS_M_SB(mapping))->min_hot_blocks)
set_inode_flag(mapping->host, FI_HOT_DATA);
@@ -2994,13 +2996,13 @@ static int f2fs_write_cache_pages(struct address_space 
*mapping,
tag_pages_for_writeback(mapping, index, end);
done_index = index;
while (!done && !retry && (index <= end)) {
-   nr_pages = find_get_pages_range_tag(mapping, &index, end,
-   tag, F2FS_ONSTACK_PAGES, pages);
-   if (nr_pages == 0)
+   nr_folios = filemap_get_folios_tag(mapping, &index, end,
+   tag, &fbatch);
+   if (nr_folios == 0)
break;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch.folios[i];
bool need_readd;
 readd:
need_readd = false;
@@ -3017,7 +3019,7 @@ static int f2fs_write_cache_pages(struct address_space 
*mapping,
}
 
if (!f2fs_cluster_can_merge_page(&cc,
-   page->index)) {
+   folio->index)) {
ret = f2fs_write_multi_pages(&cc,
&submitted, wbc, io_type);
if (!ret)
@@ -3026,27 +3028,28 @@ static int f2fs_write_cache_pages(struct address_space 
*mapping,
}
 
if (unlikely(f2fs_cp_error(sbi)))
-   goto lock_page;
+ 

[f2fs-dev] [PATCH v4 23/23] filemap: Remove find_get_pages_range_tag()

2022-11-02 Thread Vishal Moola (Oracle)
All callers to find_get_pages_range_tag(), find_get_pages_tag(),
pagevec_lookup_range_tag(), and pagevec_lookup_tag() have been removed.

Signed-off-by: Vishal Moola (Oracle) 
---
 include/linux/pagemap.h | 10 ---
 include/linux/pagevec.h |  8 --
 mm/filemap.c| 60 -
 mm/swap.c   | 10 ---
 4 files changed, 88 deletions(-)

diff --git a/include/linux/pagemap.h b/include/linux/pagemap.h
index 28275eecb949..c83dfcbc19b3 100644
--- a/include/linux/pagemap.h
+++ b/include/linux/pagemap.h
@@ -742,16 +742,6 @@ unsigned filemap_get_folios_contig(struct address_space 
*mapping,
pgoff_t *start, pgoff_t end, struct folio_batch *fbatch);
 unsigned filemap_get_folios_tag(struct address_space *mapping, pgoff_t *start,
pgoff_t end, xa_mark_t tag, struct folio_batch *fbatch);
-unsigned find_get_pages_range_tag(struct address_space *mapping, pgoff_t 
*index,
-   pgoff_t end, xa_mark_t tag, unsigned int nr_pages,
-   struct page **pages);
-static inline unsigned find_get_pages_tag(struct address_space *mapping,
-   pgoff_t *index, xa_mark_t tag, unsigned int nr_pages,
-   struct page **pages)
-{
-   return find_get_pages_range_tag(mapping, index, (pgoff_t)-1, tag,
-   nr_pages, pages);
-}
 
 struct page *grab_cache_page_write_begin(struct address_space *mapping,
pgoff_t index);
diff --git a/include/linux/pagevec.h b/include/linux/pagevec.h
index 215eb6c3bdc9..a520632297ac 100644
--- a/include/linux/pagevec.h
+++ b/include/linux/pagevec.h
@@ -26,14 +26,6 @@ struct pagevec {
 };
 
 void __pagevec_release(struct pagevec *pvec);
-unsigned pagevec_lookup_range_tag(struct pagevec *pvec,
-   struct address_space *mapping, pgoff_t *index, pgoff_t end,
-   xa_mark_t tag);
-static inline unsigned pagevec_lookup_tag(struct pagevec *pvec,
-   struct address_space *mapping, pgoff_t *index, xa_mark_t tag)
-{
-   return pagevec_lookup_range_tag(pvec, mapping, index, (pgoff_t)-1, tag);
-}
 
 static inline void pagevec_init(struct pagevec *pvec)
 {
diff --git a/mm/filemap.c b/mm/filemap.c
index cc4be51eae5b..8ad45c2e22cd 100644
--- a/mm/filemap.c
+++ b/mm/filemap.c
@@ -2318,66 +2318,6 @@ unsigned filemap_get_folios_tag(struct address_space 
*mapping, pgoff_t *start,
 }
 EXPORT_SYMBOL(filemap_get_folios_tag);
 
-/**
- * find_get_pages_range_tag - Find and return head pages matching @tag.
- * @mapping:   the address_space to search
- * @index: the starting page index
- * @end:   The final page index (inclusive)
- * @tag:   the tag index
- * @nr_pages:  the maximum number of pages
- * @pages: where the resulting pages are placed
- *
- * Like find_get_pages_range(), except we only return head pages which are
- * tagged with @tag.  @index is updated to the index immediately after the
- * last page we return, ready for the next iteration.
- *
- * Return: the number of pages which were found.
- */
-unsigned find_get_pages_range_tag(struct address_space *mapping, pgoff_t 
*index,
-   pgoff_t end, xa_mark_t tag, unsigned int nr_pages,
-   struct page **pages)
-{
-   XA_STATE(xas, &mapping->i_pages, *index);
-   struct folio *folio;
-   unsigned ret = 0;
-
-   if (unlikely(!nr_pages))
-   return 0;
-
-   rcu_read_lock();
-   while ((folio = find_get_entry(&xas, end, tag))) {
-   /*
-* Shadow entries should never be tagged, but this iteration
-* is lockless so there is a window for page reclaim to evict
-* a page we saw tagged.  Skip over it.
-*/
-   if (xa_is_value(folio))
-   continue;
-
-   pages[ret] = &folio->page;
-   if (++ret == nr_pages) {
-   *index = folio->index + folio_nr_pages(folio);
-   goto out;
-   }
-   }
-
-   /*
-* We come here when we got to @end. We take care to not overflow the
-* index @index as it confuses some of the callers. This breaks the
-* iteration when there is a page at index -1 but that is already
-* broken anyway.
-*/
-   if (end == (pgoff_t)-1)
-   *index = (pgoff_t)-1;
-   else
-   *index = end + 1;
-out:
-   rcu_read_unlock();
-
-   return ret;
-}
-EXPORT_SYMBOL(find_get_pages_range_tag);
-
 /*
  * CD/DVDs are error prone. When a medium error occurs, the driver may fail
  * a _large_ part of the i/o request. Imagine the worst scenario:
diff --git a/mm/swap.c b/mm/swap.c
index 955930f41d20..89351b6dd149 100644
--- a/mm/swap.c
+++ b/mm/swap.c
@@ -1098,16 +1098,6 @@ void folio_batch_remove_exceptionals(struct folio_batch 
*fbatch)
fbatch->nr = j;
 }
 
-unsigned pag

[f2fs-dev] [PATCH v4 12/23] f2fs: Convert f2fs_flush_inline_data() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use a folio_batch instead of pagevec. This is in
preparation for the removal of find_get_pages_tag().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Chao Yu 
---
 fs/f2fs/node.c | 17 +
 1 file changed, 9 insertions(+), 8 deletions(-)

diff --git a/fs/f2fs/node.c b/fs/f2fs/node.c
index e8b72336c096..a2f477cc48c7 100644
--- a/fs/f2fs/node.c
+++ b/fs/f2fs/node.c
@@ -1887,17 +1887,18 @@ static bool flush_dirty_inode(struct page *page)
 void f2fs_flush_inline_data(struct f2fs_sb_info *sbi)
 {
pgoff_t index = 0;
-   struct pagevec pvec;
-   int nr_pages;
+   struct folio_batch fbatch;
+   int nr_folios;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
 
-   while ((nr_pages = pagevec_lookup_tag(&pvec,
-   NODE_MAPPING(sbi), &index, PAGECACHE_TAG_DIRTY))) {
+   while ((nr_folios = filemap_get_folios_tag(NODE_MAPPING(sbi), &index,
+   (pgoff_t)-1, PAGECACHE_TAG_DIRTY,
+   &fbatch))) {
int i;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct page *page = &fbatch.folios[i]->page;
 
if (!IS_DNODE(page))
continue;
@@ -1924,7 +1925,7 @@ void f2fs_flush_inline_data(struct f2fs_sb_info *sbi)
}
unlock_page(page);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
 }
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 15/23] f2fs: Convert last_fsync_dnode() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert to use a folio_batch instead of pagevec. This is in preparation for
the removal of find_get_pages_range_tag().

Signed-off-by: Vishal Moola (Oracle) 
---
 fs/f2fs/node.c | 19 ++-
 1 file changed, 10 insertions(+), 9 deletions(-)

diff --git a/fs/f2fs/node.c b/fs/f2fs/node.c
index 38f32b4d61dc..3e1764960a96 100644
--- a/fs/f2fs/node.c
+++ b/fs/f2fs/node.c
@@ -1515,23 +1515,24 @@ static void flush_inline_data(struct f2fs_sb_info *sbi, 
nid_t ino)
 static struct page *last_fsync_dnode(struct f2fs_sb_info *sbi, nid_t ino)
 {
pgoff_t index;
-   struct pagevec pvec;
+   struct folio_batch fbatch;
struct page *last_page = NULL;
-   int nr_pages;
+   int nr_folios;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
index = 0;
 
-   while ((nr_pages = pagevec_lookup_tag(&pvec, NODE_MAPPING(sbi), &index,
-   PAGECACHE_TAG_DIRTY))) {
+   while ((nr_folios = filemap_get_folios_tag(NODE_MAPPING(sbi), &index,
+   (pgoff_t)-1, PAGECACHE_TAG_DIRTY,
+   &fbatch))) {
int i;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct page *page = &fbatch.folios[i]->page;
 
if (unlikely(f2fs_cp_error(sbi))) {
f2fs_put_page(last_page, 0);
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
return ERR_PTR(-EIO);
}
 
@@ -1562,7 +1563,7 @@ static struct page *last_fsync_dnode(struct f2fs_sb_info 
*sbi, nid_t ino)
last_page = page;
unlock_page(page);
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
return last_page;
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 11/23] f2fs: Convert f2fs_fsync_node_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use a folio_batch instead of pagevec. This is in
preparation for the removal of find_get_pages_range_tag().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Chao Yu 
---
 fs/f2fs/node.c | 19 ++-
 1 file changed, 10 insertions(+), 9 deletions(-)

diff --git a/fs/f2fs/node.c b/fs/f2fs/node.c
index 983572f23896..e8b72336c096 100644
--- a/fs/f2fs/node.c
+++ b/fs/f2fs/node.c
@@ -1728,12 +1728,12 @@ int f2fs_fsync_node_pages(struct f2fs_sb_info *sbi, 
struct inode *inode,
unsigned int *seq_id)
 {
pgoff_t index;
-   struct pagevec pvec;
+   struct folio_batch fbatch;
int ret = 0;
struct page *last_page = NULL;
bool marked = false;
nid_t ino = inode->i_ino;
-   int nr_pages;
+   int nr_folios;
int nwritten = 0;
 
if (atomic) {
@@ -1742,20 +1742,21 @@ int f2fs_fsync_node_pages(struct f2fs_sb_info *sbi, 
struct inode *inode,
return PTR_ERR_OR_ZERO(last_page);
}
 retry:
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
index = 0;
 
-   while ((nr_pages = pagevec_lookup_tag(&pvec, NODE_MAPPING(sbi), &index,
-   PAGECACHE_TAG_DIRTY))) {
+   while ((nr_folios = filemap_get_folios_tag(NODE_MAPPING(sbi), &index,
+   (pgoff_t)-1, PAGECACHE_TAG_DIRTY,
+   &fbatch))) {
int i;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct page *page = &fbatch.folios[i]->page;
bool submitted = false;
 
if (unlikely(f2fs_cp_error(sbi))) {
f2fs_put_page(last_page, 0);
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
ret = -EIO;
goto out;
}
@@ -1821,7 +1822,7 @@ int f2fs_fsync_node_pages(struct f2fs_sb_info *sbi, 
struct inode *inode,
break;
}
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
 
if (ret || marked)
diff --git a/fs/ceph/addr.c b/fs/ceph/addr.c
index dcf701b05cc1..d2361d51db39 100644
--- a/fs/ceph/addr.c
+++ b/fs/ceph/addr.c
@@ -792,7 +792,7 @@ static int ceph_writepages_start(struct address_space 
*mapping,
struct ceph_vino vino = ceph_vino(inode);
pgoff_t index, start_index, end = -1;
struct ceph_snap_context *snapc = NULL, *last_snapc = NULL, *pgsnapc;
-   struct pagevec pvec;
+   struct folio_batch fbatch;
int rc = 0;
unsigned int wsize = i_blocksize(inode);
struct ceph_osd_request *req = NULL;
@@ -821,7 +821,7 @@ static int ceph_writepages_start(struct address_space 
*mapping,
if (fsc->mount_options->wsize < wsize)
wsize = fsc->mount_options->wsize;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
 
start_index = wbc->range_cyclic ? mapping->writeback_index : 0;
index = start_index;
@@ -869,7 +869,7 @@ static int ceph_writepages_start(struct address_space 
*mapping,
 
while (!done && index <= end) {
int num_ops = 0, op_idx;
-   unsigned i, pvec_pages, max_pages, locked_pages = 0;
+   unsigned i, nr_folios, max_pages, locked_pages = 0;
struct page **pages = NULL, **data_pages;
struct page *page;
pgoff_t strip_unit_end = 0;
@@ -879,13 +879,13 @@ static int ceph_writepages_start(struct address_space 
*mapping,
max_pages = wsize >> PAGE_SHIFT;
 
 get_more_pages:
-   pvec_pages = pagevec_lookup_range_tag(&pvec, mapping, &index,
-   end, PAGECACHE_TAG_DIRTY);
-   dout("pagevec_lookup_range_tag got %d\n", pvec_pages);
-   if (!pvec_pages && !locked_pages)
+   nr_folios = filemap_get_folios_tag(mapping, &index,
+   end, PAGECACHE_TAG_DIRTY, &fbatch);
+   dout("pagevec_lookup_range_tag got %d\n", nr_folios);
+   if (!nr_folios && !locked_pages)
break;
-   for (i = 0; i < pvec_pages && locked_pages < max_pages; i++) {
-   page = pvec.pages[i];
+   for (i = 0; i < nr_folios && locked_pages < max_pages; i++) {
+   page = &fbatch.folios[i]->page;
dout("? %p idx %lu\n", page, page->index);
if (locked_pages == 0)
lock_page(page);  /* first page */
@@ -995,7 +995,7 @@ static int ceph_writepages_start(struct address_space 
*mapping,
len = 0;
}
 
    /* note position of first page in pvec */
dout("%p will write page %p idx %lu\n",
 inode, page, page->index);
 
@@ -1005,30 +1005,30 @@ static int ceph_writepages_start(struct address_space 
*mapping,
fsc->write_congested = true;
 
pages[locked_pages++] = page;
-   pvec.pages[i] = NULL;
+   fbatch.folios[i] = NULL;
 
len += thp_size(page);
}
 
/* did we get anything? */
if (!locked_pages)
-   goto release_pvec_pages;
+   goto release_folios;
if (i) {
unsigned j, n = 0;
-   /* shift unused page to beginning of pvec */
-   for (j = 0; j < pvec_pages; j++) {
-   if (!pvec.pages[j])
+   /* shift unused page to beginning of fbatch */
+   for (j = 0; j < nr_folios; j++) {
+   if (!fbatch.folios[j])
continue;
if (n < j)
-   pvec.pages[n] = pvec.pages[j];
+   fbatch.folios[n] = fbatch.folios[j];
n++;
}
-   pvec.nr = n;
+   fbatch.nr = n;
 
-   if (pvec_pages && i == pvec_pages &&
+   if (nr_folios && i == nr_folios &&
locked_pages < max_pages) {
-   dout("reached end pvec, trying for more\n");
-   pagevec_release(&pvec);
+   dout("reached end 

[f2fs-dev] [PATCH v4 22/23] nilfs2: Convert nilfs_clear_dirty_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use folios throughout. This is in preparation for
the removal of find_get_pages_range_tag(). This change removes 2 calls
to compound_head().

Signed-off-by: Vishal Moola (Oracle) 
Acked-by: Ryusuke Konishi 
---
 fs/nilfs2/page.c | 20 ++--
 1 file changed, 10 insertions(+), 10 deletions(-)
@@ -995,7 +995,7 @@ static int ceph_writepages_start(struct address_space 
}
 
-   /* note position of first page in pvec */
+   /* note position of first page in fbatch */
dout("%p will write page %p idx %lu\n",
 inode, page, page->index);

-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 16/23] f2fs: Convert f2fs_sync_meta_pages() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Convert function to use folios throughout. This is in preparation for the
removal of find_get_pages_range_tag(). This change removes 5 calls to
compound_head().

Initially the function was checking if the previous page index is truly the
previous page i.e. 1 index behind the current page. To convert to folios and
maintain this check we need to make the check
folio->index != prev + folio_nr_pages(previous folio) since we don't know
how many pages are in a folio.

At index i == 0 the check is guaranteed to succeed, so to workaround indexing
bounds we can simply ignore the check for that specific index. This makes the
initial assignment of prev trivial, so I removed that as well.

Also modified a comment in commit_checkpoint for consistency.

Signed-off-by: Vishal Moola (Oracle) 
---
 fs/f2fs/checkpoint.c | 49 +++-
 1 file changed, 26 insertions(+), 23 deletions(-)

diff --git a/fs/f2fs/checkpoint.c b/fs/f2fs/checkpoint.c
index 0c82dae082aa..82eb26f471c5 100644
--- a/fs/f2fs/checkpoint.c
+++ b/fs/f2fs/checkpoint.c
@@ -390,59 +390,62 @@ long f2fs_sync_meta_pages(struct f2fs_sb_info *sbi, enum 
page_type type,
 {
struct address_space *mapping = META_MAPPING(sbi);
pgoff_t index = 0, prev = ULONG_MAX;
-   struct pagevec pvec;
+   struct folio_batch fbatch;
long nwritten = 0;
-   int nr_pages;
+   int nr_folios;
struct writeback_control wbc = {
.for_reclaim = 0,
};
struct blk_plug plug;
 
-   pagevec_init(&pvec);
+   folio_batch_init(&fbatch);
 
blk_start_plug(&plug);
 
-   while ((nr_pages = pagevec_lookup_tag(&pvec, mapping, &index,
-   PAGECACHE_TAG_DIRTY))) {
+   while ((nr_folios = filemap_get_folios_tag(mapping, &index,
+   (pgoff_t)-1,
+   PAGECACHE_TAG_DIRTY, &fbatch))) {
int i;
 
-   for (i = 0; i < nr_pages; i++) {
-   struct page *page = pvec.pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch.folios[i];
 
-   if (prev == ULONG_MAX)
-   prev = page->index - 1;
-   if (nr_to_write != LONG_MAX && page->index != prev + 1) 
{
-   pagevec_release(&pvec);
+   if (nr_to_write != LONG_MAX && i != 0 &&
+   folio->index != prev +
+   folio_nr_pages(fbatch.folios[i-1])) {
+   folio_batch_release(&fbatch);
goto stop;
}
 
-   lock_page(page);
+   folio_lock(folio);
 
-   if (unlikely(page->mapping != mapping)) {
+   if (unlikely(folio->mapping != mapping)) {
 continue_unlock:
-   unlock_page(page);
+   folio_unlock(folio);
continue;
}
-   if (!PageDirty(page)) {
+   if (!folio_test_dirty(folio)) {
/* someone wrote it for us */
goto continue_unlock;
}
 
-   f2fs_wait_on_page_writeback(page, META, true, true);
+   f2fs_wait_on_page_writeback(&folio->page, META,
+   true, true);
 
-   if (!clear_page_dirty_for_io(page))
+   if (!folio_clear_dirty_for_io(folio))
goto continue_unlock;
 
-   if (__f2fs_write_meta_page(page, &wbc, io_type)) {
-   unlock_page(page);
+   if (__f2fs_write_meta_page(&folio->page, &wbc,
+   io_type)) {
+   folio_unlock(folio);
break;
}
-   nwritten++;
-   prev = page->index;
+   nwritten += folio_nr_pages(folio);
+   prev = folio->index;
if (unlikely(nwritten >= nr_to_write))
break;
}
-   pagevec_release(&pvec);
+   folio_batch_release(&fbatch);
cond_resched();
}
 stop:
@@ -1398,7 +1401,7 @@ static void commit_checkpoint(struct f2fs_sb_info *sbi,
};
 
/*
 * pagevec_lookup_tag and lock_page again will take
 * some extra time. Therefore, f2fs_update_meta_pages and
 * f2fs_sync_meta_pages are combined in this function.
 */
@@ -1398,7 +1401,7 @@ static void commit_checkpoint(struct f2fs_sb_info *sbi,
 
/*
-* pagevec_lookup_tag and lock_page again will take
+* filemap_get_folios_tag and lock_page again will take
 * some extra time. Therefore, f2fs_update_meta_pages and
 * f2fs_sync_meta_pages are combined in this function.
-- 
2.38.1



___
Linux-f2fs-devel mailing list
Linux-f2fs-devel@lists.sourceforge.net
https://lists.sourceforge.net/lists/listinfo/linux-f2fs-devel


[f2fs-dev] [PATCH v4 17/23] gfs2: Convert gfs2_write_cache_jdata() to use filemap_get_folios_tag()

2022-11-02 Thread Vishal Moola (Oracle)
Converted function to use folios throughout. This is in preparation for
the removal of find_get_pgaes_range_tag(). This change removes 8 calls
to compound_head().

Also had to modify and rename gfs2_write_jdata_pagevec() to take in
and utilize folio_batch rather than pagevec and use folios rather
than pages. gfs2_write_jdata_batch() now supports large folios.

Signed-off-by: Vishal Moola (Oracle) 
---
 fs/gfs2/aops.c | 64 +++---
 1 file changed, 35 insertions(+), 29 deletions(-)

diff --git a/fs/gfs2/aops.c b/fs/gfs2/aops.c
index 05bee80ac7de..8f87c2551a3d 100644
--- a/fs/gfs2/aops.c
+++ b/fs/gfs2/aops.c
@@ -195,67 +195,71 @@ static int gfs2_writepages(struct address_space *mapping,
 }
 
 /**
  * gfs2_write_jdata_pagevec - Write back a pagevec's worth of pages
  * @mapping: The mapping
  * @wbc: The writeback control
- * @pvec: The vector of pages
- * @nr_pages: The number of pages to write
+ * @fbatch: The batch of folios
  * @done_index: Page index
  *
  * Returns: non-zero if loop should terminate, zero otherwise
  */
 
-static int gfs2_write_jdata_pagevec(struct address_space *mapping,
+static int gfs2_write_jdata_batch(struct address_space *mapping,
struct writeback_control *wbc,
-   struct pagevec *pvec,
-   int nr_pages,
+   struct folio_batch *fbatch,
pgoff_t *done_index)
 {
struct inode *inode = mapping->host;
struct gfs2_sbd *sdp = GFS2_SB(inode);
-   unsigned nrblocks = nr_pages * (PAGE_SIZE >> inode->i_blkbits);
+   unsigned nrblocks;
int i;
int ret;
+   int nr_pages = 0;
+   int nr_folios = folio_batch_count(fbatch);
+
+   for (i = 0; i < nr_folios; i++)
+   nr_pages += folio_nr_pages(fbatch->folios[i]);
+   nrblocks = nr_pages * (PAGE_SIZE >> inode->i_blkbits);
 
ret = gfs2_trans_begin(sdp, nrblocks, nrblocks);
if (ret < 0)
return ret;
 
-   for(i = 0; i < nr_pages; i++) {
-   struct page *page = pvec->pages[i];
+   for (i = 0; i < nr_folios; i++) {
+   struct folio *folio = fbatch->folios[i];
 
-   *done_index = page->index;
+   *done_index = folio->index;
 
-   lock_page(page);
+   folio_lock(folio);
 
-   if (unlikely(page->mapping != mapping)) {
+   if (unlikely(folio->mapping != mapping)) {
 continue_unlock:
-   unlock_page(page);
+   folio_unlock(folio);
continue;
}
 
-   if (!PageDirty(page)) {
+   if (!folio_test_dirty(folio)) {
/* someone wrote it for us */
goto continue_unlock;
}
 
-   if (PageWriteback(page)) {
+   if (folio_test_writeback(folio)) {
if (wbc->sync_mode != WB_SYNC_NONE)
-   wait_on_page_writeback(page);
+   folio_wait_writeback(folio);
else
goto continue_unlock;
}
 
-   BUG_ON(PageWriteback(page));
-   if (!clear_page_dirty_for_io(page))
+   BUG_ON(folio_test_writeback(folio));
+   if (!folio_clear_dirty_for_io(folio))
goto continue_unlock;
 
trace_wbc_writepage(wbc, inode_to_bdi(inode));
 
-   ret = __gfs2_jdata_writepage(page, wbc);
+   ret = __gfs2_jdata_writepage(&folio->page, wbc);
if (unlikely(ret)) {
if (ret == AOP_WRITEPAGE_ACTIVATE) {
-   unlock_page(page);
+   folio_unlock(folio);
ret = 0;
} else {
 
@@ -268,7 +272,8 @@ static int gfs2_write_jdata_pagevec(struct address_space 
*mapping,
 * not be suitable for data integrity
 * writeout).
 */
-   *done_index = page->index + 1;
+   *done_index = folio->index +
+   folio_nr_pages(folio);
ret = 1;
break;
}
@@ -305,8 +310,8 @@ static int gfs2_write_cache_jdata(struct address_space 
*mapping,
 {
int ret = 0;
int done = 0;
-   struct pagevec pvec;
-   int nr_pages;
+   struct folio_batch fbatch;
+   int nr_folios;
pgoff_t writeback_index;
pgoff_t index;
pgoff_t end;
@@ -315,7 +320,7 @@ static int gfs2_write_cache_
@@ -195,67 +195,71 @@ static int gfs2_writepages(struct address_space *mapping,
 
 /**
- * gfs2_write_jdata_pagevec - Write back a pagevec's worth of pages
+ * gfs2_write_jdata_batch - Write back a folio batch's worth of folios
  * @mapping: The mapping
  * @wbc: The writeback control